@@ -18,6 +18,10 @@ from fontTools.pens.filterPen import DecomposingFilterPointPen
 from fontTools.pens.reverseContourPen import ReverseContourPen
 from fontTools.pens.transformPen import TransformPen
 
+# we keep this unused import in case some external code relied on the
+# old location of this function, which now lives in fontTools
+from fontTools.ttLib.tables.O_S_2f_2 import calcCodePageRanges  # noqa: F401
+
 from ufo2ft.constants import OPENTYPE_CATEGORIES_KEY, UNICODE_SCRIPT_ALIASES
 from ufo2ft.errors import InvalidDesignSpaceData, InvalidFontData
 from ufo2ft.fontInfoData import getAttrWithFallback
@@ -373,101 +377,6 @@ def unicodeScriptDirection(uv):
     return unicodedata.script_horizontal_direction(sc, "LTR")
 
 
-def calcCodePageRanges(unicodes):
-    """Given a set of Unicode codepoints (integers), calculate the
-    corresponding OS/2 CodePage range bits.
-    This is a direct translation of FontForge implementation:
-    https://github.com/fontforge/fontforge/blob/7b2c074/fontforge/tottf.c#L3158
-    """
-    codepageRanges = set()
-
-    chars = [chr(u) for u in unicodes]
-
-    hasAscii = set(range(0x20, 0x7E)).issubset(unicodes)
-    hasLineart = "┤" in chars
-
-    for char in chars:
-        if char == "Þ" and hasAscii:
-            codepageRanges.add(0)  # Latin 1
-        elif char == "Ľ" and hasAscii:
-            codepageRanges.add(1)  # Latin 2: Eastern Europe
-            if hasLineart:
-                codepageRanges.add(58)  # Latin 2
-        elif char == "Б":
-            codepageRanges.add(2)  # Cyrillic
-            if "Ѕ" in chars and hasLineart:
-                codepageRanges.add(57)  # IBM Cyrillic
-            if "╜" in chars and hasLineart:
-                codepageRanges.add(49)  # MS-DOS Russian
-        elif char == "Ά":
-            codepageRanges.add(3)  # Greek
-            if hasLineart and "½" in chars:
-                codepageRanges.add(48)  # IBM Greek
-            if hasLineart and "√" in chars:
-                codepageRanges.add(60)  # Greek, former 437 G
-        elif char == "İ" and hasAscii:
-            codepageRanges.add(4)  # Turkish
-            if hasLineart:
-                codepageRanges.add(56)  # IBM turkish
-        elif char == "א":
-            codepageRanges.add(5)  # Hebrew
-            if hasLineart and "√" in chars:
-                codepageRanges.add(53)  # Hebrew
-        elif char == "ر":
-            codepageRanges.add(6)  # Arabic
-            if "√" in chars:
-                codepageRanges.add(51)  # Arabic
-            if hasLineart:
-                codepageRanges.add(61)  # Arabic; ASMO 708
-        elif char == "ŗ" and hasAscii:
-            codepageRanges.add(7)  # Windows Baltic
-            if hasLineart:
-                codepageRanges.add(59)  # MS-DOS Baltic
-        elif char == "₫" and hasAscii:
-            codepageRanges.add(8)  # Vietnamese
-        elif char == "ๅ":
-            codepageRanges.add(16)  # Thai
-        elif char == "エ":
-            codepageRanges.add(17)  # JIS/Japan
-        elif char == "ㄅ":
-            codepageRanges.add(18)  # Chinese: Simplified chars
-        elif char == "ㄱ":
-            codepageRanges.add(19)  # Korean wansung
-        elif char == "央":
-            codepageRanges.add(20)  # Chinese: Traditional chars
-        elif char == "곴":
-            codepageRanges.add(21)  # Korean Johab
-        elif char == "♥" and hasAscii:
-            codepageRanges.add(30)  # OEM Character Set
-        # TODO: Symbol bit has a special meaning (check the spec), we need
-        # to confirm if this is wanted by default.
-        # elif chr(0xF000) <= char <= chr(0xF0FF):
-        #    codepageRanges.add(31)          # Symbol Character Set
-        elif char == "þ" and hasAscii and hasLineart:
-            codepageRanges.add(54)  # MS-DOS Icelandic
-        elif char == "╚" and hasAscii:
-            codepageRanges.add(62)  # WE/Latin 1
-            codepageRanges.add(63)  # US
-        elif hasAscii and hasLineart and "√" in chars:
-            if char == "Å":
-                codepageRanges.add(50)  # MS-DOS Nordic
-            elif char == "é":
-                codepageRanges.add(52)  # MS-DOS Canadian French
-            elif char == "õ":
-                codepageRanges.add(55)  # MS-DOS Portuguese
-
-    if hasAscii and "‰" in chars and "∑" in chars:
-        codepageRanges.add(29)  # Macintosh Character Set (US Roman)
-
-    # when no codepage ranges can be enabled, fall back to enabling bit 0
-    # (Latin 1) so that the font works in MS Word:
-    # https://github.com/googlei18n/fontmake/issues/468
-    if not codepageRanges:
-        codepageRanges.add(0)
-
-    return codepageRanges
-
-
 class _LazyFontName:
     def __init__(self, font):
         self.font = font
