@@ -1009,6 +1009,21 @@ class ColrCpalTest:
             "c": [("c.color2", 1), ("c.color1", 0)],
         }
 
+    def test_colr_cpal_gid1_not_blank(self, FontClass, caplog):
+        # https://github.com/MicrosoftDocs/typography-issues/issues/346
+        testufo = FontClass(getpath("ColorTest.ufo"))
+        del testufo["space"]
+
+        with caplog.at_level(logging.WARNING, logger="ufo2ft.outlineCompiler"):
+            ttf = compileTTF(testufo)
+
+        assert ttf["COLR"].version == 0
+        assert ttf.getGlyphOrder()[1] == "a"
+        assert (
+            "COLRv0 might not render correctly on Windows because "
+            "the glyph at index 1 is not empty ('a')."
+        ) in caplog.text
+
     @pytest.mark.parametrize("compileFunc", [compileTTF, compileOTF])
     @pytest.mark.parametrize("manualClipBoxes", [True, False])
     @pytest.mark.parametrize(
@@ -1482,6 +1497,22 @@ def test_MATH_table_invalid(FontClass, compile, attribute):
         compile(ufo)
 
 
+@pytest.mark.parametrize("compile", [compileTTF, compileOTF])
+def test_CMAP_format_14_table_no_cmap_entry(FontClass, compile):
+    #
+    # https://github.com/googlefonts/ufo2ft/issues/908
+    #
+    # Typically, code points listed in the uvsMappings array will have corresponding
+    # entries in a Unicode 'cmap' subtable. This is not required, however.
+    # In Bug908.ufo, the font maps U+20122 U+FE00 to U+2F803, but U+20122
+    # is not in the 'cmap' subtable.
+    ufo = FontClass(getpath("Bug908.ufo"))
+    result = compile(ufo)
+
+    cmap = result.getBestCmap()
+    assert 0x20122 not in cmap
+
+
 if __name__ == "__main__":
     import sys
 
